/*
 * ReversiAI.java
 *
 * Created on 14 de julio de 2005, 17:11
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

//package jreversi.pkg;

import java.awt.*;
import java.util.*;

/**
 *
 * @author Jaime Prez, Toms Aguado, ngel Jara
 */
public class ReversiAI {
    public static final byte EMPTY = 0;
    public static final byte WHITE = 1;
    public static final byte BLACK = 2;
    
    private static int [][] weights = {
        {120, -40, 20,  5,  5, 20, -40, 120},
        {-40, -60, -5, -5, -5, -5, -60, -40},
        { 20,  -5, 15,  3,  3, 15,  -5,  20},
        {  5,  -5,  3,  3,  3,  3,  -5,   5},
        {  5,  -5,  3,  3,  3,  3,  -5,   5},
        { 20,  -5, 15,  3,  3, 15,  -5,  20},
        {-40, -60, -5, -5, -5, -5, -60, -40},
        {120, -40, 20,  5,  5, 20, -40, 120}
    };
    
    private byte plys;
    private byte twidth;
    private byte theight;
    private byte [][] currentBoard;
    private byte turn;
    private Point humanMove;
    private boolean singlePlayer = true;
    private ReversiGUI gui;

    /** Creates a new instance of ReversiAI
     *  By default, a 8x8 board and the most simple
     *  depth for the algorithm is used.
     */
    public ReversiAI() {
        ReversiAIbuilder((byte)8, (byte)8, (byte)-1);
    }
    
    
    /** Creates a new instance of ReversiAI with
     *  a heightxwidth board and p depth for the
     *  algorithm.
     */
    public void ReversiAIbuilder(byte height, byte width, byte p) {
        plys = p;
        twidth = width;
        theight = height;
        turn = ReversiAI.WHITE;
        
        currentBoard = new byte[twidth][theight];
        for (int i = 0; i < twidth; i++)
            for (int j = 0; j < theight; j++)
                currentBoard[i][j] = ReversiAI.EMPTY;
        
        currentBoard[3][3] = ReversiAI.WHITE;
        currentBoard[3][4] = ReversiAI.BLACK;
        currentBoard[4][3] = ReversiAI.BLACK;
        currentBoard[4][4] = ReversiAI.WHITE;
        
    }
    
    /** Returns a table with the current game status
     */
    public byte[][] getCurrentStatus() {
        return currentBoard;
    }
    
    
    /** Returns board height
     */
    public int getHeight() {
        return this.theight;
    }
    
    /** Returns board width
     */
    public int getWidth() {
        return this.twidth;
    }
    
    /** Returns the "plys" used in minimax algorithm by the AI
     */
    public byte getPlys() {
        return this.plys;
    }
    
    public void setHumanMove(Point p){
        this.humanMove = p;
    }
    
    public Point getHumanMove(){
        return this.humanMove;
    }
    
    /** Returns the current turn
     */
    public byte getTurn() {
        return this.turn;
    }
    
    /** Changes player's turn to the next one
     */
    public void changeTurn() {
        if (this.turn == ReversiAI.WHITE)
            this.turn = ReversiAI.BLACK;
        else
            this.turn = ReversiAI.WHITE;
    }
    
    /** Set the "plys" to use by the AI
     */
    public void setPlys(byte p) {
        this.plys = p;
    }
    
    public boolean isSinglePlayer(){
        return this.singlePlayer;
    }
    
    public void setSinglePlayer(boolean p) {
        this.singlePlayer = p;
    }

    public void setGui(ReversiGUI g) {
        this.gui = g;
    }

    /*Dado un estado (tablero,turno) la funcin EXPANDIR
      nos va a calcular las osibles jugadas legales
      a partir del mismo.
     */
    private ArrayList expandir(byte [][] board,byte turn) {
        ArrayList moves = new ArrayList();
        for (byte i = 0; i < theight; i++){
            for (byte j = 0; j < twidth; j++){
                if (board[i][j]==ReversiAI.EMPTY){
                    //isValid hace un recorrido, abortando en cuanto encuentre
                    //un recorrido vlido devuelve TRUE con lo cual
                    //podemos meter un movimiento vlido en la lista
                    //con poco coste de computacin, los volteos efectivo para llegar al
                    //nodo desde el que se expande se realizan antes de llamar a expandir
                    if (isValidTableMove(board,turn,i,j)){
                        moves.add(new Point(i,j));
                    }
                }
            }
        }
        return moves;
    }
    
    public boolean isValidMove(byte row, byte col) {
        return isValidTableMove(currentBoard, turn, row,col);
    }
    
    private boolean isValidTableMove(byte [][] board, byte turn, byte row, byte col) {
        LinkedHashSet s = new LinkedHashSet();
        if (board[row][col] == ReversiAI.EMPTY) {  // only empty positions can be used!
            for (byte y = (byte)((col > 0) ? col - 1 : col);
            y <= ((col < (twidth-1)) ? col + 1 : col); y++) {
                for (byte x = (byte)((row>0) ? row - 1 : row);
                x <= ((row < (theight - 1)) ? row + 1 : row); x++) {
                    if ((board[x][y] != turn) && (board[x][y] != ReversiAI.EMPTY)) {
                        if (linealSearch(board,row,col,x,y,turn,s))
                            return true;
                    }
                }
            }
        }
        return false;
    }
    
    private LinkedHashSet getTakes(byte [][] board, byte turn, byte row, byte col) {
        
        LinkedHashSet s = new LinkedHashSet();
        
        if (board[row][col] == ReversiAI.EMPTY) {  // only empty positions can be used!
            for (byte y = (byte)((col > 0) ? col - 1 : col);
            y <= ((col < (twidth-1)) ? col + 1 : col); y++) {
                for (byte x = (byte)((row>0) ? row - 1 : row);
                x <= ((row < (theight - 1)) ? row + 1 : row); x++) {
                    if ((board[x][y] != turn) && (board[x][y] != ReversiAI.EMPTY)) {
                        linealSearch(board,row,col,x,y,turn,s);
                    }
                    
                }
            }
        }
        return s;
        
    }
    
    private boolean linealSearch(byte [][] board,byte ox,byte oy, byte dx,byte dy, byte t,LinkedHashSet s) {
        LinkedHashSet ns = new LinkedHashSet();
        int vx = dx - ox;
        int vy = dy - oy;
        int i = 0;
        Point p;
        
        dy = (byte)(dy + vy);
        dx = (byte)(dx + vx);
        while ((dx >= 0) && (dy >= 0) &&
                (dx <= (theight -1)) &&
                (dy <= (twidth -1))) {
            
            if (board[dx][dy] == t) {
                p = new Point(dx - vx,dy - vy);
                ns.add(p);
                s.addAll(ns);
                return true;
            }
            
            if (board[dx][dy] == ReversiAI.EMPTY)
                return false;
            
            // !t && !empty
            p = new Point(dx - vx,dy - vy);
            ns.add(p);
            
            
            dy = (byte)(dy + vy);
            dx = (byte)(dx + vx);
        }
        return false;
    }
    
    
    /** returns whether or not a player could make at least one legal
     *  move
     */
    public boolean canMakeMove() {
        
        return turnCanMakeMove(currentBoard,turn);
        
    }
    
    public boolean turnCanMakeMove(byte [][] board, byte turn) {
        
        for (byte i = 0; i < theight; i++){
            for (byte j = 0; j < twidth; j++){
                if (board[i][j]==ReversiAI.EMPTY){
                    //isValid hace un recorrido, abortando en cuanto encuentre
                    //un recorrido vlido devuelve TRUE con lo cual
                    //podemos meter un movimiento vlido en la lista
                    //con poco coste de computacin, los volteos efectivo para llegar al
                    //nodo desde el que se expande se realizan antes de llamar a expandir
                    if (isValidTableMove(board,turn,i,j)){
                        return true;
                    }
                }
            }
        }
        return false;
    }
    
    /** Update the game tree to reflect the current status
     *  when a move was done in (x,y).
     */
    public LinkedHashSet update(byte x, byte y) {
        LinkedHashSet s = getTakes(currentBoard,turn, x,y);
        currentBoard = doMove(currentBoard,turn, new Point(x, y), s);
        return s;
    }
    
    /** Suggest the best possible move given the current game status
     */
    public Point suggest() {
        int sc ,bestSc = Integer.MIN_VALUE;
        ArrayList l;
        Iterator i;
        Point move,best;
        best = new Point(-1, -1);
        l = expandir(currentBoard, ReversiAI.BLACK);
        if (l.isEmpty()){
            return new Point(-1,-1);
            //This point must never be reached because we are sure
            //AI can make a move, so "L" will never be empty
        }
        i = l.iterator();
        while (i.hasNext()) {
            move = (Point)(i.next());
            LinkedHashSet takes = getTakes(currentBoard, ReversiAI.BLACK, (byte)move.x,(byte)move.y);
            byte [][] board = doMove(currentBoard, ReversiAI.BLACK, move,takes);
            sc = minimax(board,ReversiAI.BLACK, 1, Integer.MIN_VALUE,Integer.MAX_VALUE);
            if (sc>=bestSc){
                bestSc = sc;
                best = move;
            }
            
        }
        return best;
    }
    
    
    private int max(int a, int b){
        if (a>b)
            return a;
        return b;
    }
    
    private int min(int a, int b){
        if (a<b)
            return a;
        return b;
    }
    
    public byte[][] cloneTable(byte[][] table) {
        byte[][] t = new byte[twidth][theight];
        
        for (byte x = 0; x < theight; x++)
            for (byte y = 0; y < twidth; y++)
                t[x][y] = table[x][y];
        
        return t;
    }
    
    
    private byte[][] doMove(byte[][] board,byte turn,Point move,LinkedHashSet takes){
        byte x,y;
        byte [][] t = cloneTable(board);
        
        if (turn == ReversiAI.WHITE) {
            t[move.x][move.y] = ReversiAI.WHITE;
        } else {
            t[move.x][move.y] = ReversiAI.BLACK;
        }
        Iterator it = takes.iterator();
        while (it.hasNext()) {
            Point p = (Point)it.next();
            x = (byte)p.getX();
            y = (byte)p.getY();
            if (t[x][y] == ReversiAI.WHITE)
                t[x][y] = ReversiAI.BLACK;
            else
                t[x][y] = ReversiAI.WHITE;
        }
        return t;
    }
    
    
    
    
    public byte invertTurn(byte turn) {
        if (turn == ReversiAI.WHITE)
            return  ReversiAI.BLACK;
        else
            return  ReversiAI.WHITE;
    }

    private int minimax(byte [][] board,byte turn, int depth,int alpha, int beta) {
        byte currentTurn = turn;
        ArrayList sons = new ArrayList();
        LinkedHashSet s;
        Iterator i;
        Point move;
        int v, score;
        int ex =0;
        byte [][] newBoard;

        if (depth>=plys)
            return e(board,false);

        // Itentamos expandir para el turno que toca
        // si no podemos intentamos para el otro turno llamando
        // a la funcin contraria para intentar haces dos movimientos
        // seguidos
        sons = expandir(board, currentTurn);
        //Si no pude expandir intento con el otro turno
        if (sons.isEmpty()){
            currentTurn = invertTurn(currentTurn);
            sons = expandir(board, currentTurn);
        }
        if (sons.isEmpty()){
            //No one can move, the game has ended
            if (gui.getProgress() < 100/(depth+1))
                gui.setProgress(100/(depth+1));
            return e(board,true);
        }
        i = sons.iterator();
        while (i.hasNext()) {
            move =(Point)(i.next());
            s = getTakes(board, currentTurn, (byte)move.x,(byte)move.y);
            newBoard = doMove(board, currentTurn, move, s);
            
            ex = minimax(newBoard, invertTurn(currentTurn), depth+1, alpha, beta);
            if (currentTurn == ReversiAI.BLACK){
                //This is a MAX node so we must take the highest value
                //fron the child we take a child
                if (ex > alpha)
                    alpha = ex;
                if (alpha >= beta){
                    return beta;
                }
            } else {
                //Poda
                if (ex < beta){
                    beta = ex;
                }
                if (beta<=alpha)
                    return alpha;
            }
        }

        if (gui.getProgress() < 100/(depth+1))
            gui.setProgress(100/(depth+1));

        if (currentTurn == ReversiAI.BLACK)
            return alpha;
        else
            return beta;
        
    }
    
    public boolean isFinal(byte[][] board,byte cntwhite,byte cntblack,boolean F){
        byte cntfichas =0;
        ArrayList sons = new ArrayList();
        if (F)
            return true;
        if ((cntwhite + cntblack) == 64)
            return true;
        if ((!turnCanMakeMove(board, ReversiAI.BLACK))
        &&(!turnCanMakeMove(board, ReversiAI.WHITE))){
            return true;
        }
        return false;
    }
    
    /* Evaluation function for a board and a player.
     * It uses the global "turn" to determine who
     * is the evaluation for. The evaluation cosists
     * in adding the player pieces weights and subtracts
     * the opponent's
     */
    private int e(byte [][] board,boolean F){
        
        byte cntwhite= 0,cntblack = 0;
        int count = 0;
        int w;
        
        for (byte i = 0; i < theight; i++){
            for(byte j = 0; j < twidth; j++){
                if (board[i][j]!=ReversiAI.EMPTY){
                    if (((( i == 0 && j == 1 ) ||
                            ( i == 1 && j == 0 ) ||
                            ( i == 1 && j == 1 )) &&
                            board[0][0]!= ReversiAI.EMPTY ) ||
                            ((( i == 0 && j == 6 ) ||
                            ( i == 1 && j == 6 ) ||
                            ( i == 1 && j == 7 )) &&
                            board[0][7] != ReversiAI.EMPTY ) ||
                            ((( i == 6 && j == 0 ) ||
                            ( i == 6 && j == 1 ) ||
                            ( i == 7 && j == 1 )) &&
                            board[7][0] != ReversiAI.EMPTY ) ||
                            ((( i == 6 && j == 7 ) ||
                            ( i == 6 && j == 6 ) ||
                            ( i == 7 && j == 6 )) &&
                            board[7][7] != ReversiAI.EMPTY ) )
                        w = ( 5 - weights[i][j] ) * ((board[i][j] == ReversiAI.BLACK)?1:-1);
                    else
                        w = weights[i][j];
                    if (board[i][j] == ReversiAI.BLACK){
                        count +=w;
                        cntblack++;
                    } else if (board[i][j] == ReversiAI.WHITE){
                        count -=w;
                        cntwhite++;
                    }
                }
            }
        }
        if (isFinal(board, cntwhite,cntblack,F)){
            //The AI detected a final situation in
            //possible in the future
            if (cntblack>cntwhite)
                return Integer.MAX_VALUE;
            else if (cntblack<cntwhite)
                return Integer.MIN_VALUE;
            else
                return 0;
        }
        return count;
    }
    
    
    
    public void tableDebug(byte [][] t){
        for (byte x = 0;x <twidth; x++){
            for (byte y = 0;y <theight; y++)
                System.out.print(t[x][y]);
            System.out.println();
        }
    }
    
}
