/*
 * ReversiGUI.java
 *
 * Created on 9 de julio de 2005, 16:18
 */

//package jreversi.pkg;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.table.*;
import java.util.*;
import java.text.*;

/**
 *
 * @author  Jaime Prez, Toms Aguado, ngel Jara
 */
public class ReversiGUI extends javax.swing.JFrame {
    
    private static final byte MAX_LEVEL = 12;
    
    /** The table model is used to represent data in the table.
     *  This class extends the functionality of the default one by
     *  substituting the objects in the cells by images which can be
     *  changed in runtime.
     */
    class ReversiTableModel extends javax.swing.table.AbstractTableModel {
        // basic states
        private ImageIcon empty = new ImageIcon("img/empty.gif","empty");
        private ImageIcon black = new ImageIcon("img/black.gif","black");
        private ImageIcon white = new ImageIcon("img/white.gif","white");
        // for common animations
        private ImageIcon unscaled_w2b = new ImageIcon("img/w2b.gif","black");
        private ImageIcon unscaled_b2w = new ImageIcon("img/b2w.gif", "white");
        private ImageIcon w2b;
        private ImageIcon b2w;
        
        private String[] columnNames = {"","","","","","","",""};
        private Object[][] data = {
            {empty,empty,empty,empty,empty,empty,empty,empty},
            {empty,empty,empty,empty,empty,empty,empty,empty},
            {empty,empty,empty,empty,empty,empty,empty,empty},
            {empty,empty,empty,white,black,empty,empty,empty},
            {empty,empty,empty,black,white,empty,empty,empty},
            {empty,empty,empty,empty,empty,empty,empty,empty},
            {empty,empty,empty,empty,empty,empty,empty,empty},
            {empty,empty,empty,empty,empty,empty,empty,empty}
        };
        
        public int getColumnCount() {
            return data.length;
        }
        
        public int getRowCount() {
            return data.length;
        }
        
        public String getColumnName(int col) {
            return columnNames[col];
        }
        
        public Object getValueAt(int row, int col) {
            return data[row][col];
        }
        
        public Class getColumnClass(int c) {
            return getValueAt(0, c).getClass();
        }
        
        public boolean isCellEditable(int row, int col) {
            return false;
        }
        
        public void setValueAt(Object value, int row, int col) {
            data[row][col] = value;
            fireTableCellUpdated(row,col);
        }
        
        public void setWhite(int row, int col) {
            int width = getWidth() / data.length /2;
            if (((ImageIcon)data[row][col]).getDescription() == "empty") {
                setValueAt(white,row,col);
            } else if (((ImageIcon)data[row][col]).getDescription() == "black") {
                b2w = new ImageIcon(unscaled_b2w.getImage().getScaledInstance(width, width,
                        Image.SCALE_FAST));
                b2w.setImageObserver(iobs);
                b2w.setDescription("white");
                setValueAt(b2w,row,col);
            }
            //setValueAt(white,row,col);
        }
        
        public void setBlack(int row, int col) {
            int width = getWidth() / data.length /2;
            if (((ImageIcon)data[row][col]).getDescription() == "empty") {
                setValueAt(black,row,col);
            } else if (((ImageIcon)data[row][col]).getDescription() == "white") {
                w2b = new ImageIcon(unscaled_w2b.getImage().getScaledInstance(width, width,
                        Image.SCALE_FAST));
                w2b.setImageObserver(iobs);
                w2b.setDescription("black");
                setValueAt(w2b,row,col);
            }
            //setValueAt(black,row,col);
        }
        
        public void scaleImages() {
            int width = getWidth() / data.length /2;
            empty.setImage(empty.getImage().getScaledInstance(width, width,
                    Image.SCALE_FAST));
            black.setImage(black.getImage().getScaledInstance(width, width,
                    Image.SCALE_FAST));
            white.setImage(white.getImage().getScaledInstance(width, width,
                    Image.SCALE_FAST));
            b2w = new ImageIcon(unscaled_b2w.getImage().getScaledInstance(width, width,
                    Image.SCALE_FAST));
            b2w.setImageObserver(iobs);
            w2b = new ImageIcon(unscaled_w2b.getImage().getScaledInstance(width, width,
                    Image.SCALE_FAST));
            w2b.setImageObserver(iobs);
        }
        
        public boolean isEmpty(int row, int col) {
            return ((ImageIcon)data[row][col]).getDescription() == "empty";
        }
        
        public boolean isBlack(int row, int col) {
            return ((ImageIcon)data[row][col]).getDescription() == "black";
        }
        
        public boolean isWhite(int row, int col) {
            return ((ImageIcon)data[row][col]).getDescription() == "white";
        }
        
        public void reset() {
            for (int i = 0; i < getRowCount(); i++)
                for (int j = 0; j < getColumnCount(); j++)
                    data[i][j] = empty;
            data[3][3] = white;
            data[3][4] = black;
            data[4][3] = black;
            data[4][4] = white;
            
        }
        
        public ImageObserver iobs = new ImageObserver() {
            public boolean imageUpdate(Image img, int flags, int x, int y, int w, int h) {
                repaint();
                return true;
            }};
    };
    
    /** Creates new form ReversiGUI */
    public ReversiGUI(ReversiAI rai) {
        ReversiTableModel model = new ReversiTableModel();
        AI = rai;
        
        initComponents();
        this.setLocationRelativeTo(null);
        setVisible(true);
        timeListener = new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                /****************************** This is a dirty hack!
                 * Date and Time methods seem to need negative values
                 * to represent the first hour, so an effective value
                 * of 1000 miliseconds translates into an hour and 1
                 * second, wherever a single second was expected.
                 */
                java.sql.Time time = new java.sql.Time(t -1000*60*60);
                t += 1000;
                TimeLabel.setText(time.toString());
            }
        };
        GameTimer = new javax.swing.Timer(1000,timeListener);
        PlayerDialog.setSize(200,160);
        
        model.scaleImages();
        Board.setModel(model);
        Board.setRowHeight(Board.getWidth() / Board.getColumnModel().getColumnCount());
        initBoard();
    }
    
    /** This method is called at the begining and when user requests
     *  a new game to reset and initialize the board.
     */
    private void initBoard() {
        ReversiTableModel model;
        
        model = (ReversiTableModel)Board.getModel();
        
        /* Initialize board */
        model.reset();
        Board.setEnabled(false);
        ComputerScore.setText("0");
        PlayerScore.setText("0");
    }
    
    private byte readLevel(){
        
        // level selection
        try {
            Object[] levels = new Object[MAX_LEVEL];
            for (byte i = 0; i < MAX_LEVEL; i++)
                levels[i] = (new Integer(i+1)).toString();
            String s = (String)(JOptionPane.showInputDialog(null,
                    "Select your preferred dificulty level:",
                    "Dificulty level",
                    JOptionPane.PLAIN_MESSAGE,
                    null,levels,"5"));
            if (s == null)
                return -1;
            return  (byte)(Integer.parseInt(s));
        } catch (NumberFormatException e){ return (byte)5; }
    }
    
    public void newGame(){
        byte plys = 1;
        boolean singlePlayer = true;
        
        initBoard();

        // number of players selection
        Object[] numplayers = {"Single player", "Two players"};
        String s =(String)JOptionPane.showInputDialog(null,
                "Select the game mode you want to play:",
                "Game mode",
                JOptionPane.PLAIN_MESSAGE,
                null,numplayers,"Single player");
        if (s == "Two players") {
            singlePlayer = false;
            jLabel1.setText("Player 2");
            jComboBox1.setEnabled(true);
            if (jLabel2.getText() == "Player")
                jLabel2.setText("Player 1");
        } else if (s == "Single player") {
            singlePlayer = true;
            jLabel1.setText("Computer");
            jComboBox1.setEnabled(false);
        } else return;
        
        if (singlePlayer) {
            plys = readLevel();
            if (plys == -1)
                return;
            jLabel3.setText("Level "+ plys);
        }
        AI.ReversiAIbuilder((byte)8,(byte)8,plys);
        AI.setSinglePlayer(singlePlayer);

        Board.setEnabled(true);
        
        ComputerScore.setText("2");
        PlayerScore.setText("2");
        
        //timer
        t = 1000;
        GameTimer.restart();
        
        // paint turn colours
        changeTurn(ReversiAI.WHITE);
        
        
    }
    
    public void setNewGameStarted(boolean v){
        newgamestarted = v;
    }
    
    public boolean isNewGameStarted(){
        return newgamestarted;
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        PlayerDialog = new javax.swing.JDialog();
        AcceptNameButton = new javax.swing.JButton();
        jLabel23 = new javax.swing.JLabel();
        playerName = new javax.swing.JTextField();
        jComboBox1 = new javax.swing.JComboBox();
        jPanel1 = new javax.swing.JPanel();
        Board = new javax.swing.JTable();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        jLabel9 = new javax.swing.JLabel();
        jLabel10 = new javax.swing.JLabel();
        jLabel11 = new javax.swing.JLabel();
        jLabel12 = new javax.swing.JLabel();
        jLabel13 = new javax.swing.JLabel();
        jLabel14 = new javax.swing.JLabel();
        jLabel15 = new javax.swing.JLabel();
        jLabel16 = new javax.swing.JLabel();
        jLabel17 = new javax.swing.JLabel();
        jLabel18 = new javax.swing.JLabel();
        jLabel19 = new javax.swing.JLabel();
        jLabel20 = new javax.swing.JLabel();
        jLabel21 = new javax.swing.JLabel();
        jLabel22 = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jPanel3 = new javax.swing.JPanel();
        ComputerScore = new javax.swing.JLabel();
        jPanel4 = new javax.swing.JPanel();
        PlayerScore = new javax.swing.JLabel();
        TimeLabel = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jSeparator1 = new javax.swing.JSeparator();
        jProgressBar1 = new javax.swing.JProgressBar();
        jMenuBar1 = new javax.swing.JMenuBar();
        GameMenu = new javax.swing.JMenu();
        MenuNew = new javax.swing.JMenuItem();
        MenuPlayer = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JSeparator();
        MenuExit = new javax.swing.JMenuItem();
        HelpMenu = new javax.swing.JMenu();
        MenuAbout = new javax.swing.JMenuItem();

        PlayerDialog.getContentPane().setLayout(new java.awt.GridBagLayout());

        PlayerDialog.setTitle("Change name");
        PlayerDialog.setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        PlayerDialog.setResizable(false);
        AcceptNameButton.setText("Accept");
        AcceptNameButton.setMinimumSize(new java.awt.Dimension(50, 10));
        AcceptNameButton.setPreferredSize(new java.awt.Dimension(50, 10));
        AcceptNameButton.setRequestFocusEnabled(false);
        AcceptNameButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                AcceptNameButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.ipadx = 50;
        gridBagConstraints.ipady = 15;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 30, 20, 30);
        PlayerDialog.getContentPane().add(AcceptNameButton, gridBagConstraints);

        jLabel23.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel23.setText("Please, enter your name:");
        jLabel23.setMinimumSize(new java.awt.Dimension(100, 10));
        jLabel23.setPreferredSize(new java.awt.Dimension(100, 10));
        jLabel23.setRequestFocusEnabled(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.ipadx = 70;
        gridBagConstraints.ipady = 10;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(20, 10, 10, 10);
        PlayerDialog.getContentPane().add(jLabel23, gridBagConstraints);

        playerName.setText(jLabel2.getText());
        playerName.setMinimumSize(new java.awt.Dimension(100, 10));
        playerName.setPreferredSize(new java.awt.Dimension(100, 10));
        playerName.setRequestFocusEnabled(false);
        playerName.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                playerNameActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.ipadx = 70;
        gridBagConstraints.ipady = 10;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 10, 10);
        PlayerDialog.getContentPane().add(playerName, gridBagConstraints);

        jComboBox1.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Player 1", "Player 2" }));
        jComboBox1.setEnabled(false);
        jComboBox1.setRequestFocusEnabled(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.ipadx = 70;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 10, 10);
        PlayerDialog.getContentPane().add(jComboBox1, gridBagConstraints);

        getContentPane().setLayout(new java.awt.GridBagLayout());

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("jReversi");
        setName("MainFrame");
        setResizable(false);
        jPanel1.setLayout(null);

        jPanel1.setBorder(new javax.swing.border.TitledBorder("Board"));
        jPanel1.setName("BoardPanel");
        Board.setBackground(new java.awt.Color(0, 92, 0));
        Board.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.LOWERED));
        Board.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4", "Title 5", "Title 6", "Title 7", "Title 8"
            }
        ) {
            boolean[] canEdit = new boolean [] {
                false, false, false, false, false, false, false, false
            };

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        Board.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF);
        Board.setAutoscrolls(false);
        Board.setDoubleBuffered(true);
        Board.setEnabled(false);
        Board.setGridColor(new java.awt.Color(0, 51, 0));
        Board.setIntercellSpacing(new java.awt.Dimension(0, 0));
        Board.setMaximumSize(new java.awt.Dimension(400, 400));
        Board.setMinimumSize(new java.awt.Dimension(400, 400));
        Board.setName("Panel");
        Board.setPreferredSize(new java.awt.Dimension(250, 250));
        Board.setRowSelectionAllowed(false);
        Board.setSelectionBackground(new java.awt.Color(255, 255, 255));
        Board.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                BoardMouseClicked(evt);
            }
        });

        jPanel1.add(Board);
        Board.setBounds(20, 30, 250, 250);

        jLabel5.setFont(new java.awt.Font("Courier", 1, 10));
        jLabel5.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel5.setText("A    B    C    D    E    F    G    H");
        jLabel5.setFocusable(false);
        jLabel5.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jPanel1.add(jLabel5);
        jLabel5.setBounds(20, 10, 250, 20);

        jLabel6.setFont(new java.awt.Font("Courier", 1, 10));
        jLabel6.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel6.setText("A    B    C    D    E    F    G    H");
        jLabel6.setFocusable(false);
        jLabel6.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jPanel1.add(jLabel6);
        jLabel6.setBounds(20, 280, 250, 20);

        jLabel7.setFont(new java.awt.Font("Courier", 1, 10));
        jLabel7.setText("1");
        jPanel1.add(jLabel7);
        jLabel7.setBounds(275, 35, 10, 20);

        jLabel8.setFont(new java.awt.Font("Courier", 1, 10));
        jLabel8.setText("2");
        jPanel1.add(jLabel8);
        jLabel8.setBounds(275, 66, 10, 20);

        jLabel9.setFont(new java.awt.Font("Courier", 1, 10));
        jLabel9.setText("3");
        jPanel1.add(jLabel9);
        jLabel9.setBounds(275, 97, 10, 20);

        jLabel10.setFont(new java.awt.Font("Courier", 1, 10));
        jLabel10.setText("4");
        jPanel1.add(jLabel10);
        jLabel10.setBounds(275, 128, 10, 20);

        jLabel11.setFont(new java.awt.Font("Courier", 1, 10));
        jLabel11.setText("5");
        jPanel1.add(jLabel11);
        jLabel11.setBounds(275, 159, 10, 20);

        jLabel12.setFont(new java.awt.Font("Courier", 1, 10));
        jLabel12.setText("6");
        jPanel1.add(jLabel12);
        jLabel12.setBounds(275, 190, 10, 20);

        jLabel13.setFont(new java.awt.Font("Courier", 1, 10));
        jLabel13.setText("7");
        jPanel1.add(jLabel13);
        jLabel13.setBounds(275, 221, 10, 20);

        jLabel14.setFont(new java.awt.Font("Courier", 1, 10));
        jLabel14.setText("8");
        jPanel1.add(jLabel14);
        jLabel14.setBounds(275, 252, 10, 20);

        jLabel15.setFont(new java.awt.Font("Courier", 1, 10));
        jLabel15.setText("8");
        jPanel1.add(jLabel15);
        jLabel15.setBounds(9, 252, 10, 20);

        jLabel16.setFont(new java.awt.Font("Courier", 1, 10));
        jLabel16.setText("7");
        jPanel1.add(jLabel16);
        jLabel16.setBounds(9, 221, 10, 20);

        jLabel17.setFont(new java.awt.Font("Courier", 1, 10));
        jLabel17.setText("6");
        jPanel1.add(jLabel17);
        jLabel17.setBounds(9, 190, 10, 20);

        jLabel18.setFont(new java.awt.Font("Courier", 1, 10));
        jLabel18.setText("5");
        jPanel1.add(jLabel18);
        jLabel18.setBounds(9, 159, 10, 20);

        jLabel19.setFont(new java.awt.Font("Courier", 1, 10));
        jLabel19.setText("4");
        jPanel1.add(jLabel19);
        jLabel19.setBounds(9, 128, 10, 20);

        jLabel20.setFont(new java.awt.Font("Courier", 1, 10));
        jLabel20.setText("3");
        jPanel1.add(jLabel20);
        jLabel20.setBounds(9, 97, 10, 20);

        jLabel21.setFont(new java.awt.Font("Courier", 1, 10));
        jLabel21.setText("2");
        jPanel1.add(jLabel21);
        jLabel21.setBounds(9, 66, 10, 20);

        jLabel22.setFont(new java.awt.Font("Courier", 1, 10));
        jLabel22.setText("1");
        jPanel1.add(jLabel22);
        jLabel22.setBounds(9, 35, 10, 20);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.ipadx = 289;
        gridBagConstraints.ipady = 299;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        getContentPane().add(jPanel1, gridBagConstraints);

        jPanel2.setLayout(null);

        jPanel2.setBorder(new javax.swing.border.TitledBorder("Score"));
        jPanel2.setName("ScorePanel");
        jLabel1.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel1.setText("Computer");
        jPanel2.add(jLabel1);
        jLabel1.setBounds(10, 20, 130, 20);

        jLabel2.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel2.setText("Player");
        jPanel2.add(jLabel2);
        jLabel2.setBounds(10, 90, 130, 20);

        jPanel3.setLayout(null);

        jPanel3.setBackground(new java.awt.Color(0, 0, 0));
        jPanel3.setBorder(new javax.swing.border.LineBorder(new java.awt.Color(0, 0, 0), 1, true));
        ComputerScore.setBackground(new java.awt.Color(0, 0, 0));
        ComputerScore.setFont(new java.awt.Font("Courier", 0, 14));
        ComputerScore.setForeground(new java.awt.Color(255, 255, 255));
        ComputerScore.setText("0");
        jPanel3.add(ComputerScore);
        ComputerScore.setBounds(4, 6, 30, 20);

        jPanel2.add(jPanel3);
        jPanel3.setBounds(55, 50, 40, 30);

        jPanel4.setLayout(null);

        jPanel4.setBackground(new java.awt.Color(255, 255, 255));
        jPanel4.setBorder(new javax.swing.border.LineBorder(new java.awt.Color(0, 0, 0), 1, true));
        PlayerScore.setFont(new java.awt.Font("Courier", 0, 14));
        PlayerScore.setText("0");
        jPanel4.add(PlayerScore);
        PlayerScore.setBounds(4, 6, 30, 20);

        jPanel2.add(jPanel4);
        jPanel4.setBounds(55, 120, 40, 30);

        TimeLabel.setFont(new java.awt.Font("Courier", 1, 14));
        TimeLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        TimeLabel.setText("00:00:00");
        jPanel2.add(TimeLabel);
        TimeLabel.setBounds(5, 274, 140, 20);

        jLabel3.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jPanel2.add(jLabel3);
        jLabel3.setBounds(10, 220, 130, 20);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.ipady = 100;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        getContentPane().add(jPanel2, gridBagConstraints);

        jSeparator1.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.RAISED));
        jSeparator1.setName("Separator");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.ipadx = 289;
        gridBagConstraints.ipady = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        getContentPane().add(jSeparator1, gridBagConstraints);

        jProgressBar1.setName("ProgressBar");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.ipadx = 289;
        gridBagConstraints.ipady = 6;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 0, 2, 0);
        getContentPane().add(jProgressBar1, gridBagConstraints);

        GameMenu.setMnemonic('G');
        GameMenu.setText("Game");
        GameMenu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                GameMenuActionPerformed(evt);
            }
        });

        MenuNew.setMnemonic('N');
        MenuNew.setText("New Game");
        MenuNew.setName("MenuNew");
        MenuNew.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                MenuNewActionPerformed(evt);
            }
        });

        GameMenu.add(MenuNew);

        MenuPlayer.setMnemonic('P');
        MenuPlayer.setText("Player name");
        MenuPlayer.setName("MenuPlayer");
        MenuPlayer.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                MenuPlayerActionPerformed(evt);
            }
        });

        GameMenu.add(MenuPlayer);

        GameMenu.add(jSeparator2);

        MenuExit.setMnemonic('Q');
        MenuExit.setText("Quit");
        MenuExit.setName("MenuExit");
        MenuExit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                MenuExitActionPerformed(evt);
            }
        });

        GameMenu.add(MenuExit);

        jMenuBar1.add(GameMenu);

        HelpMenu.setMnemonic('H');
        HelpMenu.setText("Help");
        MenuAbout.setMnemonic('A');
        MenuAbout.setText("About");
        MenuAbout.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                HelpAbout(evt);
            }
        });

        HelpMenu.add(MenuAbout);

        jMenuBar1.add(HelpMenu);

        setJMenuBar(jMenuBar1);

        pack();
    }
    // </editor-fold>//GEN-END:initComponents
    
    private void GameMenuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_GameMenuActionPerformed
// TODO add your handling code here:
    }//GEN-LAST:event_GameMenuActionPerformed
    
    private void HelpAbout(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_HelpAbout
        JOptionPane.showMessageDialog(null,
                "jReversi developers are:\n" +
                "* Toms Aguado Gmez\n" +
                "* Angel Jara Gmez\n" +
                "* Jaime Prez Crespo",
                "About jReversi", JOptionPane.INFORMATION_MESSAGE);
        
    }//GEN-LAST:event_HelpAbout
    
    
    private void MenuNewActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_MenuNewActionPerformed
//        System.out.println("menu new ejecutado");
        newgamestarted = true;
    }//GEN-LAST:event_MenuNewActionPerformed
    
    private void playerNameActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_playerNameActionPerformed
        
        if (playerName.getText().length() != 0) {
            if (jComboBox1.getSelectedItem() == "Player 1")
                jLabel2.setText(playerName.getText());
            else
                jLabel1.setText(playerName.getText());
            
            PlayerDialog.hide();
        }
    }//GEN-LAST:event_playerNameActionPerformed
    
    private void AcceptNameButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_AcceptNameButtonActionPerformed
        if (playerName.getText().length() != 0) {
            if (jComboBox1.getSelectedItem() == "Player 1")
                jLabel2.setText(playerName.getText());
            else
                jLabel1.setText(playerName.getText());
            
            PlayerDialog.hide();
        }
    }//GEN-LAST:event_AcceptNameButtonActionPerformed
    
    private void MenuPlayerActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_MenuPlayerActionPerformed
        playerName.setText(jLabel2.getText());
        PlayerDialog.setLocationRelativeTo(this);
        PlayerDialog.show();
    }//GEN-LAST:event_MenuPlayerActionPerformed
    
    private void MenuExitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_MenuExitActionPerformed
        System.exit(0);
    }//GEN-LAST:event_MenuExitActionPerformed
    
    public void doMove(int row, int col,LinkedHashSet s) {
        ReversiTableModel m = (ReversiTableModel)Board.getModel();
        int score;
        int t = AI.getTurn();
        
        if (t == ReversiAI.WHITE) {
            score = Integer.parseInt(PlayerScore.getText());
            PlayerScore.setText(Integer.toString(++score));
            m.setWhite(row,col);
        } else {
            score = Integer.parseInt(ComputerScore.getText());
            ComputerScore.setText(Integer.toString(++score));
            m.setBlack(row,col);
        }
        Iterator i = s.iterator();
        while (i.hasNext()) {
            Point p = (Point)i.next();
            if (m.isWhite((int)p.getX(),(int)p.getY())) {
                score = Integer.parseInt(ComputerScore.getText());
                ComputerScore.setText(Integer.toString(++score));
                score = Integer.parseInt(PlayerScore.getText());
                PlayerScore.setText(Integer.toString(--score));
                m.setBlack((int)p.getX(),(int)p.getY());
            } else {
                score = Integer.parseInt(PlayerScore.getText());
                PlayerScore.setText(Integer.toString(++score));
                score = Integer.parseInt(ComputerScore.getText());
                ComputerScore.setText(Integer.toString(--score));
                m.setWhite((int)p.getX(),(int)p.getY());
            }
        }
        
        setToolTip(row, col);
    }
    
    private void setToolTip(int col, int row) {
        char c = 'A';
        
        c = (char)((int) c + col);
        Board.setToolTipText("Last move: "
                +Character.toString(c)
                +"/"+String.valueOf(row+1));
    }

    public void setProgress(int n) {
        jProgressBar1.setValue(n);
    }

    public int getProgress() {
        return jProgressBar1.getValue();
    }

    public void changeTurn(int turn) {
        // change the current turn in the GUI
        if (turn == ReversiAI.WHITE) {
            jLabel2.setForeground(Color.RED);
            jLabel1.setForeground(Color.BLACK);
        } else {
            jLabel1.setForeground(Color.RED);
            jLabel2.setForeground(Color.BLACK);
        }
    }
    
    public void finish() {
        int black = Integer.parseInt(ComputerScore.getText());
        int white = Integer.parseInt(PlayerScore.getText());
        
        GameTimer.stop();
        jLabel1.setForeground(Color.BLACK);
        jLabel2.setForeground(Color.BLACK);
        Board.setEnabled(false);
        if (black > white)
            JOptionPane.showMessageDialog(null,"Game over, "+
                    jLabel1.getText()+" wins!","Game over", JOptionPane.YES_OPTION);
        else
            JOptionPane.showMessageDialog(null,"Game over, "+
                    jLabel2.getText()+" wins!","Game over", JOptionPane.YES_OPTION);
    }
    
    private void BoardMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_BoardMouseClicked
        int col = Board.getSelectedColumn();
        int row = Board.getSelectedRow();
        Point p;
        int t;
        
        // board active only if in human player's turn
        if ((Board.isEnabled())&&((AI.getTurn() == ReversiAI.WHITE)||(!AI.isSinglePlayer()))) {
            AI.setHumanMove(new Point(row,col));
            jProgressBar1.setValue(0);
        }
    }//GEN-LAST:event_BoardMouseClicked
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton AcceptNameButton;
    private javax.swing.JTable Board;
    private javax.swing.JLabel ComputerScore;
    private javax.swing.JMenu GameMenu;
    private javax.swing.JMenu HelpMenu;
    private javax.swing.JMenuItem MenuAbout;
    private javax.swing.JMenuItem MenuExit;
    private javax.swing.JMenuItem MenuNew;
    private javax.swing.JMenuItem MenuPlayer;
    private javax.swing.JDialog PlayerDialog;
    private javax.swing.JLabel PlayerScore;
    private javax.swing.JLabel TimeLabel;
    private javax.swing.JComboBox jComboBox1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel15;
    private javax.swing.JLabel jLabel16;
    private javax.swing.JLabel jLabel17;
    private javax.swing.JLabel jLabel18;
    private javax.swing.JLabel jLabel19;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel20;
    private javax.swing.JLabel jLabel21;
    private javax.swing.JLabel jLabel22;
    private javax.swing.JLabel jLabel23;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JProgressBar jProgressBar1;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JTextField playerName;
    // End of variables declaration//GEN-END:variables
    private javax.swing.Timer GameTimer;
    private long t;
    private ActionListener timeListener;
    
    private boolean singlePlayer = true;
    private boolean newgamestarted = false;
    private ReversiAI AI;
    
}


