/*
 * GameBoard.java
 *
 * Last modified August 8, 2005
 *
 * Generic 2 player games board with custom dimensions.
 */

/**
 *
 * @author icarus
 */

public class GameBoard {
    private byte[][] v_board;
    
    /** Creates a new instance of GameBoard */
    public GameBoard() {
        GameBoardBuilder((byte)8,(byte)8);
    }

    public GameBoard(byte width) {
        GameBoardBuilder(width,width);
    }

    public GameBoard(byte width, byte height) {
        GameBoardBuilder(width,height);
    }

    private void GameBoardBuilder(byte width, byte height) {
        v_board = new byte[width][height];
    }

    public byte getWidth() {
        return (byte)v_board.length;
    }

    public byte getHeight() {
        if (v_board.length > 0)
            return (byte)v_board[0].length;
        else
            return 0;
    }

    public byte get(byte row, byte col) {
        return v_board[row][col];
    }

    public boolean isEmpty(byte row, byte col) {
        return v_board[row][col] == GameField.EMPTY;
    }

    public boolean isFirstPlayer(byte row, byte col) {
        return v_board[row][col] == GameField.WHITE;
    }

    public boolean isSecondPlayer(byte row, byte col) {
        return v_board[row][col] == GameField.BLACK;
    }

    public void empty(byte row, byte col) {
        v_board[row][col] = GameField.EMPTY;
    }

    public void setFirstPlayer(byte row, byte col) {
        v_board[row][col] = GameField.WHITE;
    }

    public void setSecondPlayer(byte row, byte col) {
        v_board[row][col] = GameField.BLACK;
    }

    /** Clone a board
     */
    public Object clone() {
        GameBoard b = new GameBoard(this.getWidth(), this.getHeight());

        for (byte row = 0; row < this.getHeight(); row++)
            for (byte col = 0; col < this.getWidth(); col++)
                  b.v_board[row][col] = this.v_board[row][col];
        return b;
    }

    /** Prints a board
     */
    public void print() {
        for (byte row = 0; row < this.getHeight(); row++){
            for (byte col = 0; col < this.getWidth(); col++)
                System.out.print(this.v_board[row][col]);
            System.out.println();
        }
    }

}
