/*
 * ReversiTree.java
 *
 * Created on 22 de julio de 2005, 18:11
 *
 * 
 */

/**
 *
 * @author Jaime Prez, Toms Aguado, ngel Jara
 */

import java.awt.*;
import java.util.*;

public class ReversiTree {
    private GameBoard board;
    private short score;
    private byte turn;
    private ArrayList sons;
    private byte nsons;
    private ReversiTree father;
    private LinkedHashSet takes;
    private Point move;
    private boolean isfinal;
    
    /** Creates a new instance of ReversiTree */
    public ReversiTree() {
    }

    public ReversiTree(ReversiTree r,
                       GameBoard b,
                       Set s,
                       byte width,
                       byte height,
                       byte turn,
                       Point m) {
        this.score   = 0;
        this.turn    = turn;
        this.sons    = new ArrayList();
        this.nsons   = 0;
        this.board   = b;
        this.father  = r;
        this.takes   = (LinkedHashSet)s;
        this.move    = m;
        this.isfinal = false;
    }

    public GameBoard getBoard() {
        return this.board;
    }

    public int getScore() {
        return this.score;
    }

    public ArrayList getSons() {
        return this.sons;
    }

    public ReversiTree getFather() {
        return this.father;
    }

    public LinkedHashSet getTakes(){
        return this.takes;
    }

    public byte getTurn() {
        return this.turn;
    }
    
    public Point getMove() {
        return this.move;
    }

    public boolean isLeaf() {
        return sons.isEmpty();
    }

    public void setBoard(GameBoard b) {
        this.board = b;
    }

    public void setScore(short s) {
        this.score = s;
    }
    
    public void setSons(ArrayList l) {
        this.sons = l;
    }

    public void addSon(ReversiTree r) {
        this.sons.add(r);
    }

    public void deleteSon(ReversiTree r) {
        this.sons.remove(r);
    }

    public void flushList() {
        this.sons.clear();
    }

    public boolean isFinal() {
        return this.isfinal;
    }

    public byte getNumberOfSons() {
        return this.nsons;
    }

    public void clearSons() {
        this.sons = new ArrayList();
    }

    private boolean find(Point p) {
        Iterator i = this.sons.iterator();
        while (i.hasNext()) {
            ReversiTree tree = (ReversiTree)i.next();
            if (tree.move.getX() == p.getX() &&
                tree.move.getY() == p.getY())
                return true;
        }
        return false;
    }
    
    private byte findMoves(byte turn) {
        byte x,y,next;
        for (byte i = 0; i < this.board.getHeight(); i++){
            for (byte j = 0; j < this.board.getWidth(); j++){
                if (find(new Point(i,j)))
                    continue;
                LinkedHashSet s = new LinkedHashSet();
                GameBoard b = (GameBoard)this.board.clone();

                if (this.isValidTableMove(i, j, turn, s)){
                    if (turn == GameField.WHITE) {
                        next = GameField.BLACK;
                        b.setFirstPlayer(i,j);
                    } else {
                        next = GameField.WHITE;
                        b.setSecondPlayer(i,j);
                    }
                    Iterator it = s.iterator();
                    while (it.hasNext()) {
                        Point p = (Point)it.next();
                        x = (byte)p.getX();
                        y = (byte)p.getY();
                        if (b.isFirstPlayer(x,y))
                            b.setSecondPlayer(x,y);
                        else
                            b.setFirstPlayer(x,y);
                    }
                    ReversiTree node = new ReversiTree(this,b,s,
                                                       this.board.getWidth(),
                                                       this.board.getHeight(),
                                                       next,new Point(i,j));
                    this.sons.add(node);
                    this.nsons++;
                }
            }              
        }
        if (this.sons.isEmpty())
            return GameField.EMPTY;
        return turn;
    }
    
    /** given a turn (the colour of the player),
     *  and a given state (a node in a tree), it returns
     *  the node with its possible moves as child nodes
     */
    public byte expandir(byte turn) {
        if (this.sons.isEmpty()) {
            if (turn != findMoves(turn)) {
                this.turn = (turn == GameField.WHITE) ? GameField.BLACK : GameField.WHITE;
                this.turn = findMoves(this.turn);

                if (this.turn == GameField.EMPTY)
                    this.isfinal = true;
                return this.turn;
            }
            return turn;
        } else {
            return this.turn;
        }
    }

    /** Returns whether or not a position is valid, that is, there is a possible move in
     *  at least one direction using Reversi game rules.
     */
    public boolean isValidTableMove(byte row, byte col, byte turn, Set s) {
        int i = 0;
        boolean ret = false;
        GameBoard data = this.getBoard();

        if (data.isEmpty(row,col)) {  // only empty positions can be used!
            for (byte x = (byte)((row > 0) ? row - 1 : row);
                 x <= ((row < (this.board.getHeight()-1)) ? row + 1 : row); x++) {
                for (byte y = (byte)((col>0) ? col - 1 : col);
                     y <= ((col < (this.board.getWidth() - 1)) ? col + 1 : col); y++) {
                    if ((data.get(x,y) != turn) && (!data.isEmpty(x,y))) {
                        if (linealSearch(row,col,x,y,turn,s))
                            ret = true;
                    }
                    i++;
                }
            }
        }
        return ret;
    }

    /** performs a search on a board looking for "s" in the direction
     *  of the vector defined by the origin poing (ox,oy) and the
     *  destination point (dx,dy) */
    private boolean linealSearch(byte ox, byte oy, byte dx, byte dy, byte t, Set s) {
        LinkedHashSet ns = new LinkedHashSet();
        byte vx = (byte)(dx - ox);
        byte vy = (byte)(dy - oy);
        Point p;

        dy = (byte)(dy + vy);
        dx = (byte)(dx + vx);
        while ((dx >= 0) && (dy >= 0) &&
                (dx < this.board.getHeight()) &&
                (dy < this.board.getWidth())) {

            if (this.board.get(dx,dy) == t) {
                p = new Point(dx - vx,dy - vy);
                ns.add(p);
                s.addAll(ns);
                return true;
            }

            if (this.board.get(dx,dy) == GameField.EMPTY)
                return false;

            // !t && !empty
            p = new Point(dx - vx,dy - vy);
            ns.add(p);

            dy = (byte)(dy + vy);
            dx = (byte)(dx + vx);
        }
        return false;
    }    
}
